###########################################
# Target log-densities used in the paper  #
###########################################
library(mvtnorm)

#-------- 2-dimensional Gaussian Mixture ----
#--------------------------------------------
targMean1  = c(5, 0)
targMean2  = c(15, 0)
targSigma1 = rbind(c(6.25, 0),c(0, 6.25)) 
targSigma2 = rbind(c(6.25, 0),c(0, 0.25))

# Can implement 'vectorized' operations 
log.dens.target = function(theta, mu1, mu2, sig1, sig2)
{
  log( 0.5*dmvnorm(theta,mu1,sig1) + 0.5*dmvnorm(theta,mu2,sig2) )
}




#-------- 4-dimensional Gaussian Mixture ----
#--------------------------------------------
targMean1  = c(5, 5, 0, 0)
targMean2  = c(15, 15, 0, 0)
targSigma1 = rbind( c(6.25, 0, 0, 0),
                    c(0, 6.25, 0, 0),
                    c(0, 0, 6.25, 0),
                    c(0, 0, 0, 0.01) ) 
targSigma2 = rbind( c(6.25, 0, 0, 0),
                    c(0, 6.25, 0, 0),
                    c(0, 0, 0.25, 0),
                    c(0, 0, 0, 0.01) ) 

# Can implement 'vectorized' operations 
log.dens.target = function(theta, mu1, mu2, sig1, sig2)
{
  log( 0.5*dmvnorm(theta,mu1,sig1) + 0.5*dmvnorm(theta,mu2,sig2) )
}




#-------- 20-dimensional Gaussian Mixture ----------
#---------------------------------------------------
targMean1 = rep(c(5, 5, 0, 0, 0, 0, 10, 15, 0, 0),2)
targMean2 = rep(c(10, 10, 0, 0, 0, 0, 7, 20, 0, 0),2)
tmpmat1=diag( rep(c(4,4,0.5,2,1,0.1,3,4,3,0.1),2) )
tmpmat2=diag( rep(c(4,4,2.5,2,1,2.1,3,4,0.5,0.1),2) )

targSigma1 = t(tmpmat1) %*% tmpmat1 
targSigma2 = t(tmpmat2) %*% tmpmat2 

# Can implement 'vectorized' operations 
log.dens.target = function(theta, mu1, mu2, sig1, sig2)
{
  log( 0.5*dmvnorm(theta,mu1,sig1) + 0.5*dmvnorm(theta,mu2,sig2) )
}




#------- Variance Components Model -------------------
#-----------------------------------------------------
Ydye =  matrix(
  c(1545, 1440, 1440, 1520, 1580,
    1540, 1555, 1490, 1560, 1495,
    1595, 1550, 1605, 1510, 1560,
    1445, 1440, 1595, 1465, 1545,
    1595, 1630, 1515, 1635, 1625,
    1520, 1455, 1450, 1480, 1445), nrow=6,byrow=TRUE) 

a1 = a2 = 300   #hyper-parameters
b1 = b2 = 1000  #hyper-parameters
mu0 = 0         #hyper-parameters
sigma0 = 100000 #hyper-parameters

# Not 'vectorized' version (x must be a vector of parameters)
log.dens.target = function(x, Ydye, a1, a2, b1, b2, mu0, sigma0)
{
  K=dim(Ydye)[1]  
  dim = K+3
  x[1:2]=exp(x[1:2])
  return(
    (-b1/(x[1]))+(-a1-1+1)*log(x[1])+(-b2/(x[2]))+(-a2-1+1)*log(x[2])+(-(x[3]-mu0)^2/(2*sigma0^2))+
    -K*log(sqrt(x[1])) - (K*5)*log(sqrt(x[2])) +
      sum( -(x[4:(dim)]-x[3])^2/(2*x[1]) )+
      sum(-(Ydye[1,]-x[4])^2/(2*x[2]) )+
      sum(-(Ydye[2,]-x[5])^2/(2*x[2]) )+
      sum(-(Ydye[3,]-x[6])^2/(2*x[2]) )+
      sum(-(Ydye[4,]-x[7])^2/(2*x[2]) )+
      sum(-(Ydye[5,]-x[8])^2/(2*x[2]) )+
      sum(-(Ydye[6,]-x[9])^2/(2*x[2]) )  )
} 

# 'Vectorized' version (x can be a matrix)
Ydye.vec = as.vector(Ydye)
log.dens.target = function(x, Ydye.vec, a1, a2, b1, b2, mu0, sigma0)
{
  K=dim(Ydye)[1]  
  dim = K+3
  
  if(!is.vector(x))
  {
    x = t(x)
    x[1:2,]=exp(x[1:2,])
    A1=-b1/(x[1,])+(-a1-1+1)*log(x[1,])+(-b2/(x[2,]))+(-a2-1+1)*log(x[2,]) - K/2*log(x[1,]) -K*5/2*log(x[2,]) - (x[3,]-mu0)^2/(2*sigma0^2)
    A1=A1 - rowSums( (t(x[4:9,])-x[3,])^2 )/(2*x[1,])
    Theta=NULL
    for(i in 1:5)
      Theta=rbind(Theta,x[4:9,])
    A1=A1 - colSums((Theta-Ydye.vec)^2) / (2*x[2,])
    return(A1)
  }
  else
  {
    x[1:2]=exp(x[1:2])
    A1=-b1/(x[1])+(-a1-1+1)*log(x[1])+(-b2/(x[2]))+(-a2-1+1)*log(x[2]) - K/2*log(x[1]) -K*5/2*log(x[2]) - (x[3]-mu0)^2/(2*sigma0^2)
    A1=A1 - sum( (x[4:9]-x[3])^2 )/(2*x[1])
    Theta=rep(x[4:9],5)
    A1=A1 - sum( (Theta-Ydye.vec)^2 ) / (2*x[2])
    return(A1)
  }
}




#--- "Banana-shaped" Distribution (10-dimensional) -------
#---------------------------------------------------------

B = 0.01 #parameter

# Not 'vectorized' version (x must be a vector of parameters)
log.dens.target = function(x, B)
{
 dim = length(x)
 return( -(x[1]^2)/200 - 0.5*(x[2] + B*(x[1]^2) - 100*B)^2 -0.5*sum(x[3:dim]^2) )
}

# 'Vectorized' version (x can be a matrix)
log.dens.target = function(x, B)
{ 
   if (!is.vector(x))
   {
     x = t(x); dim = dim(x)[1]
     return(-(x[1,]^2)/200 -0.5*(x[2,] + B*(x[1,]^2) - 100*B)^2 -0.5*colSums(x[3:dim,]^2))
   }
   else
   {
     dim = length(x)
     return( -(x[1]^2)/200 -0.5*(x[2] + B*(x[1]^2) - 100*B)^2 -0.5*sum(x[3:dim]^2) )
   }
}  